@@ -12,12 +12,22 @@
 """
 
 
-from typing import Optional
+from typing import Optional, Union
 
 from pyemv import ac as _ac
 from pyemv import kd as _kd
 from pyemv import sm as _sm
 
+__all__ = [
+    "VisaCVN10",
+    "VisaCVN18",
+    "InteracCVN133",
+    "MasterCardCVN16",
+    "MasterCardCVN17",
+    "MasterCardCVN20",
+    "MasterCardCVN21",
+]
+
 
 class VisaCVN10(object):
     """Cryptogram Version Number (CVN) defines Card Authentication Method (CAM).
@@ -44,9 +54,9 @@ class VisaCVN10(object):
     iss_mk_smc : bytes
         16-byte binary Issuer Master Key for Issuer Script Confidentiality.
         Has to be a valid DES key.
-    pan : bytes
+    pan : bytes or str
         ASCII Application Primary Account Number.
-    psn : bytes, optional
+    psn : bytes or str, optional
         ASCII 2-digit PAN Sequence Number (default 00).
 
     Attributes
@@ -64,12 +74,12 @@ class VisaCVN10(object):
         iss_mk_ac: bytes,
         iss_mk_smi: bytes,
         iss_mk_smc: bytes,
-        pan: bytes,
-        psn: Optional[bytes] = None,
+        pan: Union[bytes, str],
+        psn: Optional[Union[bytes, str]] = None,
     ) -> None:
         # Derive AC, SMI, and SMC ICC Master Keys for a new card
         # using option A.
-        psn = psn or b"00"
+        psn = psn or "00"
         self.icc_mk_ac = _kd.derive_icc_mk_a(iss_mk_ac, pan, psn)
         self.icc_mk_smi = _kd.derive_icc_mk_a(iss_mk_smi, pan, psn)
         self.icc_mk_smc = _kd.derive_icc_mk_a(iss_mk_smc, pan, psn)
@@ -253,22 +263,22 @@ class VisaCVN10(object):
 
     def generate_pin_change_command(
         self,
-        pin: bytes,
+        pin: Union[bytes, str],
         tag_9f26: bytes,
         tag_9f36: bytes,
-        current_pin: Optional[bytes] = None,
+        current_pin: Optional[Union[bytes, str]] = None,
     ) -> bytes:
         """Generate a PIN change command with encrypted PIN block and MAC.
 
         Parameters
         ----------
-        pin : bytes
+        pin : bytes or str
             New ASCII Personal Identification Number.
         tag_9f26 : bytes
             Binary data from tag 9F26 (Authorisation Request Cryptogram).
         tag_9f36 : bytes
             Binary data from tag 9F36 (Application Transaction Counter).
-        current_pin : bytes, optional
+        current_pin : bytes or str, optional
             Current ASCII Personal Identification Number (optional). If present
             VIS PIN block is generated using current PIN.
 
@@ -320,9 +330,9 @@ class VisaCVN18(object):
     iss_mk_smc : bytes
         16-byte binary Issuer Master Key for Issuer Script Confidentiality.
         Has to be a valid DES key.
-    pan : bytes
+    pan : bytes or str
         ASCII Application Primary Account Number.
-    psn : bytes, optional
+    psn : bytes or str, optional
         ASCII 2-digit PAN Sequence Number (default 00).
 
     Attributes
@@ -340,12 +350,12 @@ class VisaCVN18(object):
         iss_mk_ac: bytes,
         iss_mk_smi: bytes,
         iss_mk_smc: bytes,
-        pan: bytes,
-        psn: Optional[bytes] = None,
+        pan: Union[bytes, str],
+        psn: Optional[Union[bytes, str]] = None,
     ) -> None:
         # Derive AC, SMI, and SMC ICC Master Keys for a new card
         # using option B.
-        psn = psn or b"00"
+        psn = psn or "00"
         self.icc_mk_ac = _kd.derive_icc_mk_b(iss_mk_ac, pan, psn)
         self.icc_mk_smi = _kd.derive_icc_mk_b(iss_mk_smi, pan, psn)
         self.icc_mk_smc = _kd.derive_icc_mk_b(iss_mk_smc, pan, psn)
@@ -549,22 +559,22 @@ class VisaCVN18(object):
 
     def generate_pin_change_command(
         self,
-        pin: bytes,
+        pin: Union[bytes, str],
         tag_9f26: bytes,
         tag_9f36: bytes,
-        current_pin: Optional[bytes] = None,
+        current_pin: Optional[Union[bytes, str]] = None,
     ) -> bytes:
         """Generate a PIN change command with encrypted PIN block and MAC.
 
         Parameters
         ----------
-        pin : bytes
+        pin : bytes or str
             New ASCII Personal Identification Number.
         tag_9f26 : bytes
             Binary data from tag 9F26 (Authorisation Request Cryptogram).
         tag_9f36 : bytes
             Binary data from tag 9F36 (Application Transaction Counter).
-        current_pin : bytes, optional
+        current_pin : bytes or str, optional
             Current ASCII Personal Identification Number (optional). If present
             VIS PIN block is generated using current PIN.
 
@@ -617,9 +627,9 @@ class InteracCVN133(object):
     iss_mk_smc : bytes
         16-byte binary Issuer Master Key for Issuer Script Confidentiality.
         Has to be a valid DES key.
-    pan : bytes
+    pan : bytes or str
         ASCII Application Primary Account Number.
-    psn : bytes, optional
+    psn : bytes or str, optional
         ASCII 2-digit PAN Sequence Number (default 00).
 
     Attributes
@@ -637,12 +647,12 @@ class InteracCVN133(object):
         iss_mk_ac: bytes,
         iss_mk_smi: bytes,
         iss_mk_smc: bytes,
-        pan: bytes,
-        psn: Optional[bytes] = None,
+        pan: Union[bytes, str],
+        psn: Optional[Union[bytes, str]] = None,
     ) -> None:
         # Derive AC, SMI, and SMC ICC Master Keys for a new card
         # using option A.
-        psn = psn or b"00"
+        psn = psn or "00"
         self.icc_mk_ac = _kd.derive_icc_mk_a(iss_mk_ac, pan, psn)
         self.icc_mk_smi = _kd.derive_icc_mk_a(iss_mk_smi, pan, psn)
         self.icc_mk_smc = _kd.derive_icc_mk_a(iss_mk_smc, pan, psn)
@@ -841,12 +851,14 @@ class InteracCVN133(object):
             _sm.EncryptionType.MASTERCARD,
         )
 
-    def generate_pin_change_command(self, pin: bytes, tag_9f26: bytes) -> bytes:
+    def generate_pin_change_command(
+        self, pin: Union[bytes, str], tag_9f26: bytes
+    ) -> bytes:
         """Generate a PIN change command with encrypted PIN block and MAC.
 
         Parameters
         ----------
-        pin : bytes
+        pin : bytes or str
             New ASCII Personal Identification Number.
         tag_9f26 : bytes
             Binary data from tag 9F26 (Authorisation Request Cryptogram).
@@ -900,9 +912,9 @@ class MasterCardCVN16(object):
     iss_mk_smc : bytes
         16-byte binary Issuer Master Key for Issuer Script Confidentiality.
         Has to be a valid DES key.
-    pan : bytes
+    pan : bytes or str
         ASCII Application Primary Account Number.
-    psn : bytes, optional
+    psn : bytes or str, optional
         ASCII 2-digit PAN Sequence Number (default 00).
 
     Attributes
@@ -920,12 +932,12 @@ class MasterCardCVN16(object):
         iss_mk_ac: bytes,
         iss_mk_smi: bytes,
         iss_mk_smc: bytes,
-        pan: bytes,
-        psn: Optional[bytes] = None,
+        pan: Union[bytes, str],
+        psn: Optional[Union[bytes, str]] = None,
     ) -> None:
         # Derive AC, SMI, and SMC ICC Master Keys for a new card
         # using option A.
-        psn = psn or b"00"
+        psn = psn or "00"
         self.icc_mk_ac = _kd.derive_icc_mk_a(iss_mk_ac, pan, psn)
         self.icc_mk_smi = _kd.derive_icc_mk_a(iss_mk_smi, pan, psn)
         self.icc_mk_smc = _kd.derive_icc_mk_a(iss_mk_smc, pan, psn)
@@ -1138,7 +1150,7 @@ class MasterCardCVN16(object):
 
     def generate_pin_change_command(
         self,
-        pin: bytes,
+        pin: Union[bytes, str],
         tag_9f26: bytes,
         tag_9f36: bytes,
     ) -> bytes:
@@ -1146,7 +1158,7 @@ class MasterCardCVN16(object):
 
         Parameters
         ----------
-        pin : bytes
+        pin : bytes or str
             New ASCII Personal Identification Number.
         tag_9f26 : bytes
             Binary data from tag 9F26 (Authorisation Request Cryptogram).
@@ -1209,9 +1221,9 @@ class MasterCardCVN17(object):
     iss_mk_smc : bytes
         16-byte binary Issuer Master Key for Issuer Script Confidentiality.
         Has to be a valid DES key.
-    pan : bytes
+    pan : bytes or str
         ASCII Application Primary Account Number.
-    psn : bytes, optional
+    psn : bytes or str, optional
         ASCII 2-digit PAN Sequence Number (default 00).
 
     Attributes
@@ -1229,12 +1241,12 @@ class MasterCardCVN17(object):
         iss_mk_ac: bytes,
         iss_mk_smi: bytes,
         iss_mk_smc: bytes,
-        pan: bytes,
-        psn: Optional[bytes] = None,
+        pan: Union[bytes, str],
+        psn: Optional[Union[bytes, str]] = None,
     ) -> None:
         # Derive AC, SMI, and SMC ICC Master Keys for a new card
         # using option A.
-        psn = psn or b"00"
+        psn = psn or "00"
         self.icc_mk_ac = _kd.derive_icc_mk_a(iss_mk_ac, pan, psn)
         self.icc_mk_smi = _kd.derive_icc_mk_a(iss_mk_smi, pan, psn)
         self.icc_mk_smc = _kd.derive_icc_mk_a(iss_mk_smc, pan, psn)
@@ -1458,7 +1470,7 @@ class MasterCardCVN17(object):
 
     def generate_pin_change_command(
         self,
-        pin: bytes,
+        pin: Union[bytes, str],
         tag_9f26: bytes,
         tag_9f36: bytes,
     ) -> bytes:
@@ -1466,7 +1478,7 @@ class MasterCardCVN17(object):
 
         Parameters
         ----------
-        pin : bytes
+        pin : bytes or str
             New ASCII Personal Identification Number.
         tag_9f26 : bytes
             Binary data from tag 9F26 (Authorisation Request Cryptogram).
@@ -1529,9 +1541,9 @@ class MasterCardCVN20(object):
     iss_mk_smc : bytes
         16-byte binary Issuer Master Key for Issuer Script Confidentiality.
         Has to be a valid DES key.
-    pan : bytes
+    pan : bytes or str
         ASCII Application Primary Account Number.
-    psn : bytes, optional
+    psn : bytes or str, optional
         ASCII 2-digit PAN Sequence Number (default 00).
 
     Attributes
@@ -1549,12 +1561,12 @@ class MasterCardCVN20(object):
         iss_mk_ac: bytes,
         iss_mk_smi: bytes,
         iss_mk_smc: bytes,
-        pan: bytes,
-        psn: Optional[bytes] = None,
+        pan: Union[bytes, str],
+        psn: Optional[Union[bytes, str]] = None,
     ) -> None:
         # Derive AC, SMI, and SMC ICC Master Keys for a new card
         # using option A.
-        psn = psn or b"00"
+        psn = psn or "00"
         self.icc_mk_ac = _kd.derive_icc_mk_a(iss_mk_ac, pan, psn)
         self.icc_mk_smi = _kd.derive_icc_mk_a(iss_mk_smi, pan, psn)
         self.icc_mk_smc = _kd.derive_icc_mk_a(iss_mk_smc, pan, psn)
@@ -1757,7 +1769,7 @@ class MasterCardCVN20(object):
 
     def generate_pin_change_command(
         self,
-        pin: bytes,
+        pin: Union[bytes, str],
         tag_9f26: bytes,
         tag_9f36: bytes,
     ) -> bytes:
@@ -1765,7 +1777,7 @@ class MasterCardCVN20(object):
 
         Parameters
         ----------
-        pin : bytes
+        pin : bytes or str
             New ASCII Personal Identification Number.
         tag_9f26 : bytes
             Binary data from tag 9F26 (Authorisation Request Cryptogram).
@@ -1828,9 +1840,9 @@ class MasterCardCVN21(object):
     iss_mk_smc : bytes
         16-byte binary Issuer Master Key for Issuer Script Confidentiality.
         Has to be a valid DES key.
-    pan : bytes
+    pan : bytes or str
         ASCII Application Primary Account Number.
-    psn : bytes, optional
+    psn : bytes or str, optional
         ASCII 2-digit PAN Sequence Number (default 00).
 
     Attributes
@@ -1848,12 +1860,12 @@ class MasterCardCVN21(object):
         iss_mk_ac: bytes,
         iss_mk_smi: bytes,
         iss_mk_smc: bytes,
-        pan: bytes,
-        psn: Optional[bytes] = None,
+        pan: Union[bytes, str],
+        psn: Optional[Union[bytes, str]] = None,
     ) -> None:
         # Derive AC, SMI, and SMC ICC Master Keys for a new card
         # using option A.
-        psn = psn or b"00"
+        psn = psn or "00"
         self.icc_mk_ac = _kd.derive_icc_mk_a(iss_mk_ac, pan, psn)
         self.icc_mk_smi = _kd.derive_icc_mk_a(iss_mk_smi, pan, psn)
         self.icc_mk_smc = _kd.derive_icc_mk_a(iss_mk_smc, pan, psn)
@@ -2067,7 +2079,7 @@ class MasterCardCVN21(object):
 
     def generate_pin_change_command(
         self,
-        pin: bytes,
+        pin: Union[bytes, str],
         tag_9f26: bytes,
         tag_9f36: bytes,
     ) -> bytes:
@@ -2075,7 +2087,7 @@ class MasterCardCVN21(object):
 
         Parameters
         ----------
-        pin : bytes
+        pin : bytes or str
             New ASCII Personal Identification Number.
         tag_9f26 : bytes
             Binary data from tag 9F26 (Authorisation Request Cryptogram).
