@@ -6,9 +6,7 @@ import binascii as _binascii
 import hashlib as _hashlib
 from typing import Optional, Union
 
-from pyemv.tools import adjust_key_parity as _adjust_key_parity
-from pyemv.tools import encrypt_tdes_ecb as _encrypt_tdes_ecb
-from pyemv.tools import xor as _xor
+from pyemv import tools as _tools
 
 __all__ = [
     "derive_icc_mk_a",
@@ -18,7 +16,9 @@ __all__ = [
 ]
 
 
-def derive_icc_mk_a(iss_mk: bytes, pan: bytes, psn: Optional[bytes] = None) -> bytes:
+def derive_icc_mk_a(
+    iss_mk: bytes, pan: Union[bytes, str], psn: Optional[Union[bytes, str]] = None
+) -> bytes:
     r"""ICC Master Key Derivation. EMV Option A.
     Uses PAN, PAN Sequence Number, MK ISS, Triple DES.
 
@@ -27,9 +27,9 @@ def derive_icc_mk_a(iss_mk: bytes, pan: bytes, psn: Optional[bytes] = None) -> b
     iss_mk : bytes
         Binary Issuer Master Key to derive ICC Master Key from.
         Has to be a valid DES key.
-    pan : bytes
+    pan : bytes or str
         ASCII Application Primary Account Number.
-    psn : bytes, optional
+    psn : bytes or str, optional
         ASCII 2-digit PAN Sequence Number (default 00).
 
     Returns
@@ -57,26 +57,34 @@ def derive_icc_mk_a(iss_mk: bytes, pan: bytes, psn: Optional[bytes] = None) -> b
     --------
     >>> from pyemv import kd
     >>> iss_mk = bytes.fromhex("0123456789ABCDEFFEDCBA9876543210")
-    >>> icc_mk = kd.derive_icc_mk_a(iss_mk, pan=b"12345678901234567", psn=b"01")
+    >>> icc_mk = kd.derive_icc_mk_a(iss_mk, pan="12345678901234567", psn="01")
     >>> icc_mk.hex().upper()
     '73AD54688CEF2934B0979857E3C719F1'
     """
     if psn is None:
-        psn = b"00"
+        psn = "00"
+
+    if isinstance(psn, bytes):
+        psn = psn.decode("ascii")
+
+    if isinstance(pan, bytes):
+        pan = pan.decode("ascii")
 
     # Data A must be at most 16 digits, right-justified,
     # zero-padded from the left.
     data_a = _binascii.a2b_hex((pan + psn)[-16:].zfill(16))
 
     # Data B is inverted data A
-    data_b = _xor(data_a, b"\xFF" * len(data_a))
+    data_b = _tools.xor(data_a, b"\xFF" * len(data_a))
 
-    icc_mk = _encrypt_tdes_ecb(iss_mk, data_a + data_b)
+    icc_mk = _tools.encrypt_tdes_ecb(iss_mk, data_a + data_b)
 
-    return _adjust_key_parity(icc_mk)
+    return _tools.adjust_key_parity(icc_mk)
 
 
-def derive_icc_mk_b(iss_mk: bytes, pan: bytes, psn: Optional[bytes] = None) -> bytes:
+def derive_icc_mk_b(
+    iss_mk: bytes, pan: Union[bytes, str], psn: Optional[Union[bytes, str]] = None
+) -> bytes:
     r"""ICC Master Key Derivation. EMV Option B.
     Uses PAN, PAN Sequence Number, MK ISS, Triple DES, SHA-1 and
     decimalisation of hex digits.
@@ -86,9 +94,9 @@ def derive_icc_mk_b(iss_mk: bytes, pan: bytes, psn: Optional[bytes] = None) -> b
     iss_mk : bytes
         Binary Issuer Master Key to derive ICC Master Key from.
         Has to be a valid DES key.
-    pan : bytes
+    pan : bytes or str
         ASCII Application Primary Account Number.
-    psn : bytes, optional
+    psn : bytes or str, optional
         ASCII 2-digit PAN Sequence Number (default 00).
 
     Returns
@@ -116,7 +124,7 @@ def derive_icc_mk_b(iss_mk: bytes, pan: bytes, psn: Optional[bytes] = None) -> b
     --------
     >>> from pyemv import kd
     >>> iss_mk = bytes.fromhex("0123456789ABCDEFFEDCBA9876543210")
-    >>> icc_mk = kd.derive_icc_mk_b(iss_mk, pan=b"12345678901234567", psn=b"01")
+    >>> icc_mk = kd.derive_icc_mk_b(iss_mk, pan="12345678901234567", psn="01")
     >>> icc_mk.hex().upper()
     'AD406D7F6D7570916D75E5DCAB8CF737'
     """
@@ -125,12 +133,18 @@ def derive_icc_mk_b(iss_mk: bytes, pan: bytes, psn: Optional[bytes] = None) -> b
         return derive_icc_mk_a(iss_mk, pan, psn)
 
     if psn is None:
-        psn = b"00"
+        psn = "00"
+
+    if isinstance(psn, bytes):
+        psn = psn.decode("ascii")
+
+    if isinstance(pan, bytes):
+        pan = pan.decode("ascii")
 
     # Data A must be an even number of digits,
     # right-justified, zero-padded from the left.
     if len(pan) % 2:
-        pan_psn = _binascii.a2b_hex(b"0" + pan + psn)
+        pan_psn = _binascii.a2b_hex("0" + pan + psn)
     else:
         pan_psn = _binascii.a2b_hex(pan + psn)
 
@@ -146,22 +160,17 @@ def derive_icc_mk_b(iss_mk: bytes, pan: bytes, psn: Optional[bytes] = None) -> b
     # Table 0 1 2 3 4 5
     if len(result) < 16:
         digest = "".join(filter((lambda x: x in ("abcdef")), digest))
-        digest = digest.replace("a", "0")
-        digest = digest.replace("b", "1")
-        digest = digest.replace("c", "2")
-        digest = digest.replace("d", "3")
-        digest = digest.replace("e", "4")
-        digest = digest.replace("f", "5")
+        digest = digest.translate({97: 48, 98: 49, 99: 50, 100: 51, 101: 52, 102: 53})
         result = result + digest[: 16 - len(result)]
 
     data_a = _binascii.a2b_hex(result)
 
     # Data B is inverted data A
-    data_b = _xor(data_a, b"\xFF" * len(data_a))
+    data_b = _tools.xor(data_a, b"\xFF" * len(data_a))
 
-    icc_mk = _encrypt_tdes_ecb(iss_mk, data_a + data_b)
+    icc_mk = _tools.encrypt_tdes_ecb(iss_mk, data_a + data_b)
 
-    return _adjust_key_parity(icc_mk)
+    return _tools.adjust_key_parity(icc_mk)
 
 
 def derive_common_sk(icc_mk: bytes, r: Union[bytes, bytearray]) -> bytes:
@@ -221,9 +230,9 @@ def derive_common_sk(icc_mk: bytes, r: Union[bytes, bytearray]) -> bytes:
     r_b = bytearray(r)
     r_b[2] = 0x0F
 
-    sk = _encrypt_tdes_ecb(icc_mk, r_a + r_b)
+    sk = _tools.encrypt_tdes_ecb(icc_mk, r_a + r_b)
 
-    return _adjust_key_parity(sk)
+    return _tools.adjust_key_parity(sk)
 
 
 def derive_visa_sm_sk(icc_mk: bytes, atc: bytes) -> bytes:
@@ -263,12 +272,12 @@ def derive_visa_sm_sk(icc_mk: bytes, atc: bytes) -> bytes:
     if len(atc) != 2:
         raise ValueError("ATC value must be 2 bytes long")
 
-    # SK Key A (i.e. first 8 bytes) = r _xor MK Key A
+    # SK Key A (i.e. first 8 bytes) = r xor MK Key A
     r = b"\x00" * 6 + atc
-    sk_a = _xor(r, icc_mk[:8])
+    sk_a = _tools.xor(r, icc_mk[:8])
 
-    # SK Key B (i.e. second 8 bytes) = r _xor MK Key B
-    r = b"\x00" * 6 + _xor(atc, b"\xff\xff")
-    sk_b = _xor(r, icc_mk[8:])
+    # SK Key B (i.e. second 8 bytes) = r xor MK Key B
+    r = b"\x00" * 6 + _tools.xor(atc, b"\xff\xff")
+    sk_b = _tools.xor(r, icc_mk[8:])
 
-    return _adjust_key_parity(sk_a + sk_b)
+    return _tools.adjust_key_parity(sk_a + sk_b)
