@@ -7,13 +7,10 @@ using encipherment of the data field.
 import binascii as _binascii
 import sys as _sys
 from enum import Enum as _Enum
-from typing import Optional
+from typing import Optional, Union
 
-from pyemv.mac import mac_iso9797_3 as _mac_iso9797_3
-from pyemv.mac import pad_iso9797_2 as _pad_iso9797_2
-from pyemv.tools import encrypt_tdes_cbc as _encrypt_tdes_cbc
-from pyemv.tools import encrypt_tdes_ecb as _encrypt_tdes_ecb
-from pyemv.tools import xor as _xor
+from pyemv import mac as _mac
+from pyemv import tools as _tools
 
 __all__ = [
     "generate_command_mac",
@@ -88,7 +85,7 @@ def generate_command_mac(
     if len(sk_smi) != 16:
         raise ValueError("Session Key must be a double length DES key")
 
-    return _mac_iso9797_3(sk_smi[:8], sk_smi[-8:], command, 2, length)
+    return _mac.mac_iso9797_3(sk_smi[:8], sk_smi[-8:], command, 2, length)
 
 
 class EncryptionType(_Enum):
@@ -146,7 +143,7 @@ def encrypt_command_data(
 
     See Also
     --------
-    pyemv.mac._pad_iso9797_2 : ISO/IEC 9797-1 padding method 2
+    pyemv.mac.pad_iso9797_2 : ISO/IEC 9797-1 padding method 2
 
     Examples
     --------
@@ -164,9 +161,9 @@ def encrypt_command_data(
     # Prepend data length as a single byte then
     # pad according to ISO/IEC 9797-1 method 2
     if encryption_type == EncryptionType.VISA:
-        return _encrypt_tdes_ecb(
+        return _tools.encrypt_tdes_ecb(
             sk_smc,
-            _pad_iso9797_2(
+            _mac.pad_iso9797_2(
                 len(command_data).to_bytes(1, _sys.byteorder) + command_data, 8
             ),
         )
@@ -175,21 +172,21 @@ def encrypt_command_data(
     # pad according to ISO/IEC 9797-1 method 2
     if encryption_type == EncryptionType.MASTERCARD:
         if len(command_data) % 8 > 0:
-            return _encrypt_tdes_cbc(
+            return _tools.encrypt_tdes_cbc(
                 sk_smc,
                 b"\x00\x00\x00\x00\x00\x00\x00\x00",
-                _pad_iso9797_2(command_data, 8),
+                _mac.pad_iso9797_2(command_data, 8),
             )
-        return _encrypt_tdes_cbc(
+        return _tools.encrypt_tdes_cbc(
             sk_smc, b"\x00\x00\x00\x00\x00\x00\x00\x00", command_data
         )
 
     # Always pad according to ISO/IEC 9797-1 method 2
     if encryption_type == EncryptionType.EMV:
-        return _encrypt_tdes_cbc(
+        return _tools.encrypt_tdes_cbc(
             sk_smc,
             b"\x00\x00\x00\x00\x00\x00\x00\x00",
-            _pad_iso9797_2(command_data, 8),
+            _mac.pad_iso9797_2(command_data, 8),
         )
 
     raise TypeError(
@@ -200,7 +197,9 @@ def encrypt_command_data(
 
 
 def format_vis_pin_block(
-    icc_mk_ac: bytes, pin: bytes, current_pin: Optional[bytes] = None
+    icc_mk_ac: bytes,
+    pin: Union[bytes, str],
+    current_pin: Optional[Union[bytes, str]] = None,
 ) -> bytes:
     r"""Format VIS PIN block with or without the current PIN.
 
@@ -209,9 +208,9 @@ def format_vis_pin_block(
     icc_mk_ac : bytes
         Binary 16-byte ICC Master Key for Application Cryptogram
         Has to be a valid DES key.
-    pin : bytes
+    pin : bytes or str
         New ASCII Personal Identification Number.
-    current_pin : bytes, optional
+    current_pin : bytes or str, optional
         Current ASCII Personal Identification Number (optional). If present
         VIS PIN block is generated using current PIN.
 
@@ -231,7 +230,7 @@ def format_vis_pin_block(
     --------
     >>> from pyemv import sm
     >>> icc_mk_ac = bytes.fromhex("0123456789ABCDEFFEDCBA9876543210")
-    >>> sm.format_vis_pin_block(icc_mk_ac, b"1234").hex().upper()
+    >>> sm.format_vis_pin_block(icc_mk_ac, "1234").hex().upper()
     '041234FF76543210'
     """
     if len(pin) < 4 or len(pin) > 12:
@@ -240,34 +239,40 @@ def format_vis_pin_block(
     if len(icc_mk_ac) != 16:
         raise ValueError("ICC Master Key for AC must be a double length DES key")
 
+    if isinstance(pin, bytes):
+        pin = pin.decode("ascii")
+
+    if isinstance(current_pin, bytes):
+        current_pin = current_pin.decode("ascii")
+
     # 4 right-most bytes of ICC MK AC Key A padded with 0x00 to form an 8-byte block
     block_a = b"\x00" * 4 + icc_mk_ac[4:8]
 
     # PIN length as 1 byte concatenated with PIN then F-padded to form an 8 byte block
     block_b = len(pin).to_bytes(1, _sys.byteorder) + _binascii.a2b_hex(
-        pin + b"F" * (14 - len(pin))
+        pin + "F" * (14 - len(pin))
     )
 
-    pin_block = _xor(block_a, block_b)
+    pin_block = _tools.xor(block_a, block_b)
 
     # Generate VIS PIN block using current PIN
     if current_pin is not None:
         if len(current_pin) < 4 or len(current_pin) > 12:
             raise ValueError("Current PIN must be between 4 and 12 digits long")
 
-        pin_block = _xor(
-            pin_block, _binascii.a2b_hex(current_pin + b"0" * (16 - len(current_pin)))
+        pin_block = _tools.xor(
+            pin_block, _binascii.a2b_hex(current_pin + "0" * (16 - len(current_pin)))
         )
 
     return pin_block
 
 
-def format_iso9564_2_pin_block(pin: bytes) -> bytes:
+def format_iso9564_2_pin_block(pin: Union[bytes, str]) -> bytes:
     r"""Format ISO 9564-1 PIN block format 2.
 
     Parameters
     ----------
-    pin : bytes
+    pin : bytes or str
         New ASCII Personal Identification Number.
 
     Returns
@@ -283,12 +288,15 @@ def format_iso9564_2_pin_block(pin: bytes) -> bytes:
     Examples
     --------
     >>> from pyemv import sm
-    >>> sm.format_iso9564_2_pin_block(b"123456789012").hex().upper()
+    >>> sm.format_iso9564_2_pin_block("123456789012").hex().upper()
     '2C123456789012FF'
     """
     if len(pin) < 4 or len(pin) > 12:
         raise ValueError("PIN must be between 4 and 12 digits long")
 
+    if isinstance(pin, bytes):
+        pin = pin.decode("ascii")
+
     return (len(pin) + 32).to_bytes(1, _sys.byteorder) + _binascii.a2b_hex(
-        pin + b"F" * (14 - len(pin))
+        pin + "F" * (14 - len(pin))
     )
